==> fling_pickle-0.4.0.tar.gz: lib/src/pickle.dart <==
part of fling_pickle;

/// A class that can be turned into a [Pickle].
abstract class Pickleable {
  Pickle asPickle();
}

class PickleBuilder {
  /// Constructs an empty [PickleBuilder].
  PickleBuilder() : _values = {};

  /// Builds the [Pickle] representation of this.
  Pickle build() => Pickle._(_values);

  /// Places a `String` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withString(final String key, final String value) {
    _values[key] = value;
    return this;
  }

  /// Places a `bool` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withBool(final String key, final bool value) {
    _values[key] = value;
    return this;
  }

  /// Places an `int` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withInt(final String key, final int value) {
    _values[key] = value;
    return this;
  }

  /// Places a `double` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withDouble(final String key, final double value) {
    _values[key] = value;
    return this;
  }

  /// Places a `Pickle` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withPickle(final String key, final Pickle pickle) {
    _values[key] = pickle._values;
    return this;
  }

  /// Places a `Pickleable` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withPickleable(final String key, final Pickleable pickleable) =>
      withPickle(key, pickleable.asPickle());

  /// Places a `List` of [Pickle]s in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withPickles(final String key, final List<Pickle> pickles) {
    _values[key] = pickles
        .map((final Pickle pickle) => pickle._values)
        .toList(growable: false);
    return this;
  }

  /// Places a `List` of [Pickleable]s in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withPickleables(
      final String key, final Iterable<Pickleable> pickleables) {
    _values[key] = pickleables
        .map((final Pickleable pickleable) => pickleable.asPickle()._values)
        .toList(growable: false);
    return this;
  }

  final Map<String, dynamic> _values;
}

/// A Memento-like abstraction of an object's state.
class Pickle {
  /// Reads a `String` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  String readString(final String key, {final String? defaultValue}) =>
      _read(key, defaultValue: defaultValue);

  /// Reads a `bool` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  bool readBool(final String key, {final bool? defaultValue}) =>
      _read(key, defaultValue: defaultValue);

  /// Reads an `int` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  int readInt(final String key, {final int? defaultValue}) =>
      _read(key, defaultValue: defaultValue);

  /// Reads a `double` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  double readDouble(final String key, {final double? defaultValue}) =>
      _read(key, defaultValue: defaultValue);

  /// Reads a `Pickle` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  Pickle readPickle(final String key, {final Pickle? defaultValue}) {
    try {
      return Pickle._(_read(key));
    } catch (error) {
      if (null != defaultValue) {
        return defaultValue;
      }
      rethrow;
    }
  }

  /// Reads a `Pickleable` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  T readPickleable<T>(
      final String key, final T Function(Pickle pickle) depickler,
      {final T? defaultValue}) {
    try {
      return depickler(readPickle(key));
    } catch (error) {
      if (defaultValue != null) {
        return defaultValue;
      }
      rethrow;
    }
  }

  /// Reads a `List` of [Pickleable]s from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  List<T> readPickleables<T>(
    final String key,
    final T Function(Pickle pickle) depickler, {
    final List<T>? defaultValue,
  }) {
    try {
      return _readList(key)
          .map<T>((final Map<String, dynamic> map) => depickler(Pickle._(map)))
          .toList();
    } catch (error) {
      if (defaultValue != null) {
        return defaultValue;
      }
      rethrow;
    }
  }

  /// Reads a `List` of [Pickle]s from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  List<Pickle> readPickles(final String key,
      {final List<Pickle>? defaultValue}) {
    try {
      return _readList(key).map(_from).toList();
    } catch (error) {
      if (defaultValue != null) {
        return defaultValue;
      }
      rethrow;
    }
  }

  /// Attempts to read an attribute from a specific key.
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  T _read<T>(final String key, {final T? defaultValue}) {
    if (!_values.containsKey(key)) {
      if (defaultValue == null) {
        throw ArgumentError('No such key: "$key"');
      }
      return defaultValue;
    }
    dynamic value = _values[key];
    if (!(value is T)) {
      throw PickleTypeError(T, value.runtimeType);
    }
    return _values[key];
  }

  /// Reads a dynamic list into a Dart JSON list.
  List<Map<String, dynamic>> _readList(final String key) =>
      List<Map<String, dynamic>>.from(_read(key));

  /// Creates a [Pickle] from a Dart JSON object.
  static Pickle _from(final Map<String, dynamic> pickleValues) =>
      Pickle._(pickleValues);

  /// The values stored in this [Pickle].
  final Map<String, dynamic> _values;

  /// Constructs a [Pickle] from existing data.
  Pickle._(final Map<String, dynamic> pickleValues) : _values = pickleValues;

  @override
  bool operator ==(final Object? other) =>
      other is Pickle && _values == other._values;

  @override
  int get hashCode => _values.hashCode + 3;

  @override
  String toString() => _values.toString();
}

/// Error thrown when attempting to read a value from a [Pickle] that is of a
/// different type than expected.
class PickleTypeError extends TypeError {
  /// The type expected from the caller.
  final Type expected;

  /// The type in the actual [Pickle].
  final Type received;

  /// Constructs a [PickleTypeError].
  PickleTypeError(final this.expected, final this.received);

  @override
  String toString() =>
      'PickleTypeError: attempted to depickle "$received" as "$expected"';
}

==> fling_pickle-0.4.3.tar.gz: lib/src/pickle.dart <==
part of fling_pickle;

/// A class that can be turned into a [Pickle].
abstract class Pickleable {
  Pickle asPickle();
}

class PickleBuilder {
  /// Constructs an empty [PickleBuilder].
  PickleBuilder() : _values = {};

  /// Builds the [Pickle] representation of this.
  Pickle build() => Pickle._(_values);

  /// Places a `String` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withString(final String key, final String value) {
    _values[key] = value;
    return this;
  }

  /// Places a `bool` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withBool(final String key, final bool value) {
    _values[key] = value;
    return this;
  }

  /// Places an `int` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withInt(final String key, final int value) {
    _values[key] = value;
    return this;
  }

  /// Places a `double` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withDouble(final String key, final double value) {
    _values[key] = value;
    return this;
  }

  /// Places a `Pickle` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withPickle(final String key, final Pickle pickle) {
    _values[key] = pickle._values;
    return this;
  }

  /// Places a `Pickleable` in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withPickleable(final String key, final Pickleable pickleable) =>
      withPickle(key, pickleable.asPickle());

  /// Places a `List` of [Pickle]s in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withPickles(final String key, final List<Pickle> pickles) {
    _values[key] = pickles
        .map((final Pickle pickle) => pickle._values)
        .toList(growable: false);
    return this;
  }

  /// Places a `List` of [Pickleable]s in the [Pickle].
  ///
  /// This method is chainable.
  PickleBuilder withPickleables(
      final String key, final Iterable<Pickleable> pickleables) {
    _values[key] = pickleables
        .map((final Pickleable pickleable) => pickleable.asPickle()._values)
        .toList(growable: false);
    return this;
  }

  final Map<String, dynamic> _values;
}

/// A Memento-like abstraction of an object's state.
class Pickle {
  /// Reads a `String` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  String readString(final String key, {final String? defaultValue}) =>
      _read(key, defaultValue: defaultValue);

  /// Reads a `bool` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  bool readBool(final String key, {final bool? defaultValue}) =>
      _read(key, defaultValue: defaultValue);

  /// Reads an `int` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  int readInt(final String key, {final int? defaultValue}) =>
      _read(key, defaultValue: defaultValue);

  /// Reads a `double` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  double readDouble(final String key, {final double? defaultValue}) =>
      _read(key, defaultValue: defaultValue);

  /// Reads a `Pickle` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  Pickle readPickle(final String key, {final Pickle? defaultValue}) {
    try {
      return Pickle._(_read(key));
    } catch (error) {
      if (null != defaultValue) {
        return defaultValue;
      }
      rethrow;
    }
  }

  /// Reads a `Pickleable` from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  T readPickleable<T>(
      final String key, final T Function(Pickle pickle) depickler,
      {final T? defaultValue}) {
    try {
      return depickler(readPickle(key));
    } catch (error) {
      if (defaultValue != null) {
        return defaultValue;
      }
      rethrow;
    }
  }

  /// Reads a `List` of [Pickleable]s from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  List<T> readPickleables<T>(
    final String key,
    final T Function(Pickle pickle) depickler, {
    final List<T>? defaultValue,
  }) {
    try {
      return _readList(key)
          .map<T>((final Map<String, dynamic> map) => depickler(Pickle._(map)))
          .toList();
    } catch (error) {
      if (defaultValue != null) {
        return defaultValue;
      }
      rethrow;
    }
  }

  /// Reads a `List` of [Pickle]s from this [Pickle].
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  List<Pickle> readPickles(final String key,
      {final List<Pickle>? defaultValue}) {
    try {
      return _readList(key).map(_from).toList();
    } catch (error) {
      if (defaultValue != null) {
        return defaultValue;
      }
      rethrow;
    }
  }

  /// Attempts to read an attribute from a specific key.
  ///
  /// This will throw [ArgumentError] if the key is empty and no default value
  /// has been provided or [PickleTypeError] if the associated value is of a
  /// different type than expected.
  T _read<T>(final String key, {final T? defaultValue}) {
    if (!_values.containsKey(key)) {
      if (defaultValue == null) {
        throw ArgumentError('No such key: "$key"');
      }
      return defaultValue;
    }
    dynamic value = _values[key];
    if (value is! T) {
      throw PickleTypeError(T, value.runtimeType);
    }
    return _values[key];
  }

  /// Reads a dynamic list into a Dart JSON list.
  List<Map<String, dynamic>> _readList(final String key) =>
      List<Map<String, dynamic>>.from(_read(key));

  /// Creates a [Pickle] from a Dart JSON object.
  static Pickle _from(final Map<String, dynamic> pickleValues) =>
      Pickle._(pickleValues);

  /// The values stored in this [Pickle].
  final Map<String, dynamic> _values;

  /// Constructs a [Pickle] from existing data.
  Pickle._(final Map<String, dynamic> pickleValues) : _values = pickleValues;

  @override
  bool operator ==(final Object? other) =>
      other is Pickle && _values == other._values;

  @override
  int get hashCode => _values.hashCode + 3;

  @override
  String toString() => _values.toString();
}

/// Error thrown when attempting to read a value from a [Pickle] that is of a
/// different type than expected.
class PickleTypeError extends TypeError {
  /// The type expected from the caller.
  final Type expected;

  /// The type in the actual [Pickle].
  final Type received;

  /// Constructs a [PickleTypeError].
  PickleTypeError(this.expected, this.received);

  @override
  String toString() =>
      'PickleTypeError: attempted to depickle "$received" as "$expected"';
}
